﻿Public Class Etc

    Public Const D100Meg As Decimal = 100000000
    Public Const D10Meg As Decimal = 10000000
    Public Const D1Meg As Decimal = 1000000
    Public Const D100kilo As Decimal = 100000
    Public Const D10kilo As Decimal = 10000
    Public Const D1kilo As Decimal = 1000

    Public Const F100Meg As Double = 100000000.0
    Public Const F10Meg As Double = 10000000.0
    Public Const F1Meg As Double = 1000000.0
    Public Const F100kilo As Double = 100000.0
    Public Const F10kilo As Double = 10000.0
    Public Const F1kilo As Double = 1000.0

    Public Enum WShape As Integer
        Sin
        Rectangle
        Triangle
    End Enum

    '周波数値から周波数を表す文字列を生成する
    Public Shared Function GetFrequencyStr(ByVal freq As Double) As String
        If (freq > F1Meg) Then '1MHzより大はエラー
            Return "Freq. OVF"
        ElseIf (freq = F1Meg) Then '1MHzの時
            Return "1.00000MHz"
        ElseIf (freq >= F100kilo) Then '100KHz以上
            Return ((freq / F1kilo).ToString("F3") + "kHz")
        ElseIf (freq >= F10kilo) Then '10KHz以上
            Return ((freq / F1kilo).ToString("F4") + "kHz")
        ElseIf (freq >= F1kilo) Then '1KHz以上
            Return ((freq / F1kilo).ToString("F5") + "kHz")
        ElseIf (freq >= 100.0) Then '100Hz以上
            Return ((freq.ToString("F3") + "Hz"))
        ElseIf (freq >= 10.0) Then '10Hz以上
            Return ((freq.ToString("F4") + "Hz"))
        ElseIf (freq >= 1.0) Then '1Hz以上
            Return ((freq.ToString("F5") + "Hz"))
        ElseIf (freq >= 0.1) Then '0.1Hz以上
            Return ((freq.ToString("F6") + "Hz"))
        ElseIf (freq >= 0.01) Then '0.01Hz以上
            Return ((freq.ToString("F7") + "Hz"))
        Else
            Return ((freq.ToString("F7") + "Hz"))
        End If
    End Function

    ' クロックの要求周波数からカウンタの設定値を
    ' 算出する
    ' freq:要求周波数
    ' kilo:周波数を1000倍するかどうか
    ' setn:設定すべき数値
    ' 範囲外ならfalse
    Public Shared Function GetClockSetting(ByVal freq As Double, ByVal kilo As Boolean, ByRef setn As UInteger)
        If (kilo) Then
            freq *= F1kilo
        End If
        Dim tmpd As Double = Math.Round(F100Meg / freq, MidpointRounding.AwayFromZero)
        setn = 0
        If (tmpd < 100.0) Then
            Return False
        ElseIf (tmpd > 4297296925.0) Then
            Return False
        End If
        setn = tmpd
        Return True
    End Function

    '波形イメージの要求周波数から波形に必要なデータ数算出する
    Public Shared Function GetWaveLength(ByVal freq As Double, ByVal kilo As Boolean, ByVal clkfreq As Double, ByRef len As Integer)
        If (kilo) Then
            freq *= F1kilo
        End If
        '四捨五入して整数化
        len = Math.Round(clkfreq / freq, MidpointRounding.AwayFromZero)
        '波形個数が2以上、メモリ長以内になければエラー
        If ((len < 2) Or (len > 262144)) Then
            Return False
        Else
            Return True
        End If
    End Function

    'クロックの設定値から実際に生成される周波数を
    '算出する
    Public Shared Function GetActualFrequency(ByVal setn As UInteger) As Double
        If (setn = 0) Then
            Return 0.0
        End If
        Dim tmpd As Double = F100Meg / CType(setn, Double)
        Return tmpd
    End Function

    '電圧レベルがレンジ内であるかどうか確認する
    Public Shared Function VoltRangeChk(ByVal range As RangeSel, ByVal volt As Double) As Boolean
        Dim VRangeMax() As Double = 'レンジ毎の最大値
        {
            9.9997, '±10Vレンジ
            4.9998, '±5Vレンジ
            2.4999, '±2.5Vレンジ
            9.9998, '10Vレンジ
            4.9999  '5Vレンジ
        }
        Dim VRangeMin() As Double = 'レンジ毎の最小値
        {
            -10.0,  '±10Vレンジ
            -5.0,   '±5Vレンジ
            -2.5,   '±2.5Vレンジ
            0.0,    '10Vレンジ
            0.0     '5Vレンジ
        }
        Dim tmprange As Integer = CType(range, Integer)
        If (VRangeMax(tmprange) < volt) Then
            Return False
        End If
        If (VRangeMin(tmprange) > volt) Then
            Return False
        End If
        Return True
    End Function

    '電圧値(double)からDACに設定すべき値を算出する
    Public Shared Function DoubleToDacCode(ByVal dat As Double, ByVal range As RangeSel) As UShort
        Dim tmpd As Double
        Select Case (range)
            Case RangeSel.Bipol10 : tmpd = (dat + 10.0) / 20.0 * 65536.0
            Case RangeSel.Bipol5 : tmpd = (dat + 5.0) / 10.0 * 65536.0
            Case RangeSel.Bipol2_5 : tmpd = (dat + 2.5) / 5.0 * 65536.0
            Case RangeSel.Unipol10 : tmpd = dat / 10.0 * 65536.0
            Case RangeSel.Unipol5 : tmpd = dat / 5.0 * 65536.0
            Case Else : tmpd = 0.0
        End Select
        Return tmpd.ToUShort()
    End Function

    'DACの値(UShort)から相当する電圧(Double)を算出する
    Public Shared Function DacCodeToVolt(ByVal dat As UShort, ByVal range As RangeSel) As Double
        Select Case (range)
            Case RangeSel.Bipol10 : Return (dat / 65536.0 * 20.0 - 10.0)
            Case RangeSel.Bipol5 : Return (dat / 65536.0 * 10.0 - 5.0)
            Case RangeSel.Bipol2_5 : Return (dat / 65536.0 * 5.0 - 2.5)
            Case RangeSel.Unipol10 : Return (dat / 65536.0 * 10.0)
            Case RangeSel.Unipol5 : Return (dat / 65536.0 * 5.0)
            Case Else : Return 0.0
        End Select
    End Function

    '波形イメージを生成する
    Public Shared Sub WaveImageGen(ByVal shape As WShape, ByVal range As RangeSel, ByVal offset As Double, ByVal amp As Double, ByRef wimage() As UShort, ByVal len As Integer)
        Select Case (shape)
            Case WShape.Sin 'sin
                For i As Integer = 0 To (len - 1)
                    wimage(i) = DoubleToDacCode(offset + amp * Math.Sin(2 * Math.PI * CType(i, Double) / CType(len, Double)), range)
                Next
            Case WShape.Rectangle 'rectangle
                Dim HD As UShort = DoubleToDacCode(offset + amp, range)
                Dim ld As UShort = DoubleToDacCode(offset - amp, range)
                Dim mid As Integer = len / 2
                For i As Integer = 0 To (mid - 1)
                    wimage(i) = HD
                Next
                For i As Integer = mid To (len - 1)
                    wimage(i) = ld
                Next
            Case WShape.Triangle 'triangle
                Dim diff As Double = (amp * 4.0) / CType(len, Double)
                Dim Deg90 As Integer = len / 4
                Dim Deg270 As Integer = len * 3 / 4
                For i As Integer = 0 To (Deg90 - 1)
                    wimage(i) = DoubleToDacCode(offset + diff * CType(i, Double), range)
                Next
                For i As Integer = Deg90 To (Deg270 - 1)
                    wimage(i) = DoubleToDacCode(offset + 2.0 * amp - diff * CType(i, Double), range)
                Next
                For i As Integer = Deg270 To (len - 1)
                    wimage(i) = DoubleToDacCode(offset - 4.0 * amp + diff * CType(i, Double), range)
                Next
            Case Else
                For i As Integer = 0 To (len - 1)
                    wimage(i) = 0
                Next
        End Select
    End Sub
End Class
