﻿'-----------------------------------------------
'TUSB-0416DAM用ドライバアクセス用API
'Visual Basic 用モジュール
'
'2024_02_29　株式会社タートル工業
'Copyright (C) 2024 Turtle Industry Co.,Ltd.
'-----------------------------------------------

Imports System.Runtime.InteropServices
Imports TU_STATUS = System.UInt32

' 設定値定義
Public Enum ModuleN As Byte
    A = 0
    B = 1
    Ext = 2
End Enum
Public Enum GateSt As Byte
    GOff = 0
    GOn = 1
End Enum
Public Enum ChSel As Byte
    Ch1 = 0
    Ch2 = 1
    Ch3 = 2
    Ch4 = 3
End Enum
Public Enum RangeSel As Byte
    Bipol10 = 0     '±10V
    Bipol5 = 1      '±5V
    Bipol2_5 = 2    '±2.5V
    Unipol10 = 3    '10V
    Unipol5 = 4     '5V
End Enum
Public Enum TrgSel As Byte
    TStop = 0
    TStart = 1
End Enum
Public Enum TrgStat As Byte
    TStop = 0
    TRun = 1
End Enum

'DAC動作設定構造体
Public Enum SetMode As Byte '動作モード
    DC = 0
    Gate = 1
    Trg = 2
    Fifo = 3
End Enum
Public Enum SigSrc As Byte 'クロック、トリガなどのソース
    Internal = 0
    External = 1
End Enum
Public Enum SigInv As Byte '信号反転
    NonInvert = 0
    Invert = 1
End Enum
Public Enum RestartPos As Byte '波形出力モードがゲートの時にゲート停止から再びゲート開始になった時に波形メモリカウンタを何処から始めるか
    Zero = 0        'ゼロから開始
    Conti = 1       ' 前回の続き
End Enum
Public Enum FinState As Byte   '波形出力停止時の状態選択
    LastValue = 0       '最終出力値
    SpecificValue = 1   '特定値
End Enum

<StructLayout(LayoutKind.Sequential)> Public Structure stSettings
    Public Mode As SetMode      '動作モード 0:固定値出力 1:連続波形ゲート動作 2:連続波形トリガ動作
    Public ClkSel As SigSrc     '連続波形の時のクロックソース 0:内部 1:外部入力
    Public GTSel As SigSrc      '連続波形の時のゲート/トリガソース 0:内部 1:外部入力
    Public ExGtInv As SigInv    '外部ゲート信号論理反転 0:反転しない(Lowの時出力) 1:反転する ※ゲートの時のみ　トリガの場合は反転しない
    Public Rstc As RestartPos   '波形出力モードがゲートの時にゲート停止から再びゲート開始になった時に波形メモリカウンタをリセットするか　0:する 1:しない
    Public xOption As Byte      'オプション　予約機能 0を設定する
    <VBFixedArray(1), MarshalAs(UnmanagedType.ByValArray, SizeConst:=2)>
    Public FStat() As FinState  '波形出力停止時の状態選択 0:波形メモリの最終値 1:停止時出力指定値
    <VBFixedArray(1), MarshalAs(UnmanagedType.ByValArray, SizeConst:=2)>
    Public FVal() As UShort     '波形停止時出力指定値
    Public RepCyc As UShort     'トリガ動作波形出力回数
End Structure

Module TUSB0416DAM
    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Connect(
        ByVal SerialNumber As String
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_DisConnect(
    ByVal SerialNumber As String
    ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Dio_Dir(
        ByVal SerialNumber As String,
        ByVal DirData As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Dio_Dir_R(
        ByVal SerialNumber As String,
        <Out> ByRef DirData As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Dio_In(
        ByVal SerialNumber As String,
        <Out> ByRef InData As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Dio_Out(
        ByVal SerialNumber As String,
        ByVal InData As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Dio_Out_R(
        ByVal SerialNumber As String,
        <Out> ByRef OutData As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Exs_Sel(
        ByVal SerialNumber As String,
        ByVal OutSel0 As Byte,
        ByVal OutSel1 As Byte
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Clock_Gen(
        ByVal SerialNumber As String,
        ByVal MSel As ModuleN,
        ByVal DivideN As UInteger
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_DacGrSet(
        ByVal SerialNumber As String,
        ByVal MSel As ModuleN,
        <[In]> ByRef Damset As stSettings
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_DacGrSet_R(
        ByVal SerialNumber As String,
        ByVal MSel As ModuleN,
        <Out> ByRef Damset As stSettings
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_WavImgSet(
        ByVal SerialNumber As String,
        ByVal MSel As ModuleN,
        <[In]()> ByVal WImage1() As UShort,
        <[In]()> ByVal WImage2() As UShort,
        ByVal Len As Integer
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Fifo_ImgSet(
        ByVal SerialNumber As String,
        ByVal Ch As ChSel,
        <[In]()> ByVal WImage() As UShort,
        ByVal Len As Integer
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Fifo_Clr(
        ByVal SerialNumber As String,
        ByVal Ch As ChSel
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Fifo_Status(
       ByVal SerialNumber As String,
       <Out()> ByVal Over() As Byte,
       <Out()> ByVal Under() As Byte,
       <Out()> ByVal DataLen() As UInteger
       ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Trigger(
       ByVal SerialNumber As String,
       ByVal MSel As ModuleN,
       ByVal SSel As TrgSel
       ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_Trigger_Stat(
       ByVal SerialNumber As String,
       ByVal MSel As ModuleN,
       <Out> ByRef Stat As TrgStat
       ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_GateSet(
       ByVal SerialNumber As String,
       ByVal MSel As ModuleN,
       ByVal GateEn As GateSt
       ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_GateSet_R(
       ByVal SerialNumber As String,
       ByVal MSel As ModuleN,
       <Out> ByRef GateEn As GateSt
       ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_RangeSet(
        ByVal SerialNumber As String,
        ByVal Ch As ChSel,
        ByVal Range As RangeSel
        ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_RangeSet_R(
        ByVal SerialNumber As String,
        ByVal Ch As ChSel,
        <Out> ByRef Range As RangeSel
) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_DcValSet(
   ByVal SerialNumber As String,
   ByVal Ch As ChSel,
   ByVal NVal As UShort
   ) As TU_STATUS
    End Function

    <DllImport("TUSB0416DAM.DLL", CallingConvention:=CallingConvention.StdCall)>
    Function TUSB0416DAM_DcValSet_R(
   ByVal SerialNumber As String,
   ByVal Ch As ChSel,
   <Out> ByRef NVal As UShort
   ) As TU_STATUS
    End Function


    ' Error Message
    Public Function GetErrMessage(ByVal ErrCode As TU_STATUS) As String

        Select Case ErrCode
            Case 0
                Return "正常終了しました"
            Case 1
                Return "引数が不正です"
            Case 2
                Return "接続されていません"
            Case 3
                Return "接続されている台数が多すぎます"
            Case 4
                Return "デバイスがみつかりません"
            Case 5
                Return "USB通信エラーです"
            Case 6
                Return "デバイスは接続されています"
            Case Else
                Return "不明なエラーです"
        End Select
    End Function
End Module
